# flake8: noqa
PREFIX = """Отвечай человеку как можно более полезно и точно. У тебя есть доступ к следующим инструментам:"""
FORMAT_INSTRUCTIONS = """Используй json-объект для указания инструмента, предоставив ключ действия (имя инструмента) и ключ ввода действия (ввод инструмента).

Допустимые значения "action": "Final Answer" или {tool_names}

Предоставь только ОДНО действие на $JSON_BLOB, как показано:

```
{{{{
  "action": $TOOL_NAME,
  "action_input": $INPUT
}}}}
```

Следуй этому формату:

Question: вводимый вопрос для ответа
Thought: учитывай предыдущие и последующие шаги
Action:
```
$JSON_BLOB
```
Observation: результат действия
... (повторяй Thought/Action/Action Input/Observation может повторяться N раз)
Thought: Я знаю, что отвечать
Action:
```
{{{{
  "action": "Final Answer",
  "action_input": "Окончательный ответ человеку"
}}}}
```"""
SUFFIX = """Начни! Напоминаю, что ВСЕГДА нужно отвечать действительным json-объектом одного действия. Используй инструменты, если это необходимо. Отвечай напрямую, если это уместно. Формат - Action:```$JSON_BLOB```затем Observation:.
Thought:"""
