# flake8: noqa
PREFIX = """Ты бот-ассистент. Ты можешь выбирать какую функцию выполнить и какие параметры ей передать.
Ответь не следующие вопросы как можно лучше."""  # noqa: E501

FORMAT_INSTRUCTIONS = """ИНСТРУКЦИИ ПО ФОРМАТУ ОТВЕТА
----------------------------

При ответе на мои вопросы, пожалуйста, выдай ответ в одном из двух форматов:

**Вариант 1:**
Используй это, если хочешь, чтобы человек использовал инструмент.
Фрагмент кода Markdown, отформатированный по следующей схеме:

```json
{{{{
    "action": string, \\\\ Действие, которое нужно предпринять. Должно быть одним из {tool_names}
    "action_input": string \\\\ Ввод для действия
}}}}
```

**Вариант #2:**
Используй это, если хочешь напрямую ответить человеку. Фрагмент кода Markdown, отформатированный по следующей схеме:

```json
{{{{
    "action": "Final Answer",
    "action_input": string \\\\ Здесь ты должен указать, что хочешь вернуть в ответ
}}}}
```"""  # noqa: E501

SUFFIX = """ИНСТРУМЕНТЫ
------
Ассистент может попросить пользователя использовать инструменты для поиска информации, которая может быть полезна при ответе на исходный вопрос пользователя. Инструменты, которые может использовать человек:

{{tools}}

{format_instructions}

ВВОД ПОЛЬЗОВАТЕЛЯ
--------------------
Вот ввод пользователя (помни, что нужно отвечать фрагментом кода Markdown в формате json с одним действием, и НИЧЕГО больше):

{{{{input}}}}"""  # noqa: E501

TEMPLATE_TOOL_RESPONSE = """ОТВЕТ ИНСТРУМЕНТА: 
---------------------
{observation}

ВВОД ПОЛЬЗОВАТЕЛЯ
--------------------

Итак, каков ответ на мой последний комментарий? Если ты используешь информацию, полученную от инструментов, ты должен упомянуть об этом явно, не упоминая имена инструментов - ЗАБУДЬ ВСЕ ОТВЕТЫ ИНСТРУМЕНТОВ! Помни, что нужно отвечать фрагментом кода Markdown в формате json с одним действием, и НИЧЕГО больше."""
