# flake8: noqa
"""Prompt для цепочки роутеров в цепочке вопросов и ответов с множественным извлечением."""

MULTI_RETRIEVAL_ROUTER_TEMPLATE = """\
Учитывая запрос к системе ответов на вопросы, выбери наиболее подходящую систему \
для входных данных. Тебе будут даны имена доступных систем и описание \
для каких вопросов эта система наиболее подходит. Ты также можешь пересмотреть исходный \
запрос, если считаешь, что его изменение в конечном итоге приведет к лучшему ответу.

<< ФОРМАТИРОВАНИЕ >>
Верни фрагмент кода markdown с объектом JSON, отформатированным таким образом:
```json
{{{{
    "destination": string \\ имя системы ответов на вопросы для использования или "DEFAULT"
    "next_inputs": string \\ возможно измененная версия исходного запроса
}}}}
```

ПОМНИ: "destination" ДОЛЖЕН быть одним из указанных ниже имен кандидатов на промпт ИЛИ \
он может быть "DEFAULT", если входные данные не подходят для любого из кандидатов на промпт.
ПОМНИ: "next_inputs" может быть просто исходным запросом, если ты не считаешь, что нужны \
какие-либо изменения.

<< КАНДИДАТЫ НА ПРОМПТ >>
{destinations}

<< ВХОДНЫЕ ДАННЫЕ >>
{{input}}

<< ВЫХОДНЫЕ ДАННЫЕ >>
"""
