# flake8: noqa
"""Запрос для роутера в цепочке множественных запросов."""

MULTI_PROMPT_ROUTER_TEMPLATE = """\
Учитывая исходный текстовый ввод в языковую модель, выбери наиболее подходящий запрос для \
ввода. Тебе будут даны имена доступных запросов и описание того, для чего лучше всего подходит \
запрос. Ты также можешь пересмотреть исходный ввод, если считаешь, что его изменение в конечном итоге \
приведет к лучшему ответу от языковой модели.

<< ФОРМАТИРОВАНИЕ >>
Верни фрагмент кода в markdown с объектом JSON, отформатированным таким образом:
```json
{{{{
    "destination": string \\ имя используемого запроса или "DEFAULT"
    "next_inputs": string \\ возможно измененная версия исходного ввода
}}}}
```

ПОМНИ: "destination" ДОЛЖЕН быть одним из имен кандидатов на запрос, указанных ниже, ИЛИ \
он может быть "DEFAULT", если ввод не очень подходит для любого из кандидатов на запрос.
ПОМНИ: "next_inputs" может быть просто исходным вводом, если ты не думаешь, что \
необходимы какие-либо изменения.

<< КАНДИДАТЫ НА ЗАПРОСЫ >>
{destinations}

<< ВВОД >>
{{input}}

<< ВЫВОД (должен включать ```json в начале ответа) >>
"""
