# flake8: noqa
prompt_template = """Учитывая следующий вопрос и контекст, извлеките любую часть контекста *КАК ЕСТЬ*, которая имеет отношение к ответу на вопрос. Если ни одна часть контекста не имеет отношения, верните {no_output_str}. 

Помните, *НЕ РЕДАКТИРУЙТЕ* извлеченные части контекста.

> Вопрос: {{question}}
> Контекст:
>>>
{{context}}
>>>
Извлеченные релевантные части:"""
